function obj = cnv2mat(f, sx, shape)
%CNV2MAT creates a matrix object for 2d convolutions.
%
% Inputs:
%   f        the left-hand-side of the convolution
%   sx       the size of the right-hand-side of the convolution
%   shape    one of 'valid' or 'same'
%
% Example of usage:
%   F = cnv2mat(f, sx, 'same');
%   y = F*x;    % same as y = imfilter(x, f, 'same');
%
% Michael Hirsch * 20/07/2010


sf = size(f);        % size of the psf, not of the matrix

if any(sf < sx) && any(sf > sx)
  error('[%s.m] size missmatch', mfilename);
end

sz = max(sx, sf);    % the bigger one
sw = min(sx, sf);    % the smaller one

switch shape
 case 'valid'
  sy = sz - sw + 1;
 case 'same'
  sy = sz;
  sz = sx + sf - 1;
 otherwise
  error('[cnv2mat.m] Unknown shape. Choose between ''same'' and ''valid''');
end

% precompute fft already
z = fftn(f, sz);

% calculate its size
s = [prod(sy), prod(sx)];

% store all stuff in the structure
obj.s     = s;        % precomputed size
obj.shape = shape;    % shape of the convolution
obj.f     = f;        % PSF
obj.z     = z;        % precomputed FFT
obj.sx    = sx;       % input dimension
obj.sy    = sy;       % output dimension
obj.sf    = sf;       % size of the PSF
obj.sz    = sz;       % size of the FFT
obj.sw    = sw;       % size of the PSF
obj.tp    = false;    % is the resulting matrix transposed?
obj = class(obj, 'cnv2mat');
